#include <WiFi.h>
#include <PubSubClient.h>
#include "config.h"
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <ArduinoJson.h>

// define detected
int ir1 = 5;
int ir2 = 6;

// Wifi section 
WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);
uint32_t last_publish;


void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}


void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  mqtt.setCallback(mqtt_callback);
  printf("MQTT broker connected.\n");
}


void mqtt_callback(char* topic, byte* payload, unsigned int length) {
  delay(200);
}


void setup() {
  Serial.begin(115200);

  // detected
  pinMode(ir1,INPUT);
  pinMode(ir2,INPUT);

  connect_wifi();
  connect_mqtt();
  last_publish = 0;
  Serial.begin(115200);
  Wire.begin(48, 47);

}


void loop() {
  // check for incoming subscribed topics
  mqtt.loop();

  if(digitalRead(ir1) == 1){
    Serial.println("1 not Detected");
    mqtt.publish(TOPIC_PREFIX "/sensor1", "0");
    delay(200);
  }
  else{
    Serial.println("1 Detected");
    mqtt.publish(TOPIC_PREFIX "/sensor1", "1");
    delay(200);
  }

  if(digitalRead(ir2) == 1){
    Serial.println("2 not Detected");
    mqtt.publish(TOPIC_PREFIX "/sensor2", "0");
    delay(200);
  }
  else{
    Serial.println("2 Detected");
    mqtt.publish(TOPIC_PREFIX "/sensor2", "1");
    delay(200);
  }

  StaticJsonBuffer<300> JSONbuffer;
  JsonObject& JSONencoder = JSONbuffer.createObject();
 
  JSONencoder["sensor1"] = 1-digitalRead(ir1);
  JSONencoder["sensor2"] = 1-digitalRead(ir2);

  char JSONmessageBuffer[100];
  JSONencoder.printTo(JSONmessageBuffer, sizeof(JSONmessageBuffer));
  Serial.println("Sending message to MQTT topic..");
  Serial.println(JSONmessageBuffer);
  mqtt.publish(TOPIC_PREFIX "/sensor", JSONmessageBuffer);

  delay(200);

}
