#include <WiFi.h>
#include <PubSubClient.h>
#include "config.h"
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Fonts/FreeSerif9pt7b.h>
#include <pwmWrite.h>



#define gatesensor 5
#define servoPin 6
#define TOPIC_SENSOR_1 TOPIC_PREFIX "/sensor1"  //sensorตัว1 
#define TOPIC_SENSOR_2 TOPIC_PREFIX "/sensor2" //sensorตัว2
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
Pwm pwm = Pwm();

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define NUMFLAKES     10 // Number of snowflakes in the animation example

#define LOGO_HEIGHT   16
#define LOGO_WIDTH    16

WiFiClient wifiClient;
PubSubClient mqtt("iot.cpe.ku.ac.th", 1883, wifiClient);
uint32_t last_publish;

int sen1=0;
int sen2=0;


void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}


void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("iot.cpe.ku.ac.th", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  mqtt.setCallback(mqtt_callback);
  mqtt.subscribe(TOPIC_PREFIX "/sensor1");
  mqtt.subscribe(TOPIC_PREFIX "/sensor2");
  printf("MQTT broker connected.\n");
}

int state=1;

void mqtt_callback(char* topic, byte* payload, unsigned int length) {
  
 if (strcmp(topic, TOPIC_PREFIX "/sensor1") == 0) {
    payload[length] = 0; // null-terminate the payload to treat it as a string
    int value1 = atoi((char*)payload); 
    if (value1 == 1) {
      Serial.println("1detected");
    }
    else if (value1 == 0) {
      Serial.println("1undetected");
    }
    else {
      printf("Invalid payload received.\n");
    }
    sen1=value1;
  }

if (strcmp(topic, TOPIC_PREFIX "/sensor2") == 0) {
    payload[length] = 0; // null-terminate the payload to treat it as a string
    int value2 = atoi((char*)payload); 
    if (value2 == 1) {
      Serial.println("2detected");
    }
    else if (value2 == 0) {
      Serial.println("2undetected");
    }
    else {
      printf("Invalid payload received.\n");
    }
    sen2=value2;
  }

}

void setup() {

  connect_wifi();
  connect_mqtt();
  last_publish = 0;
  pinMode(gatesensor, INPUT);
  Serial.begin(115200);
  analogSetAttenuation(ADC_11db);
  Wire.begin(48, 47);  // กำหนดขา SDA => GPIO48, SCL => GPIO47
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) 
  { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }
  display.clearDisplay();
  
  delay(2000);
}


void loop() {
  mqtt.loop();

  //servo with sensor
  int val = digitalRead(gatesensor);
  if (val == 0){
    pwm.writeServo(servoPin, 90);
    delay(2000);
    pwm.writeServo(servoPin, 0);
  }

  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(22, 0);// Display static text
  display.println("Garage Legacy");
  display.setCursor(7, 20);
  display.println("Parking remains : ");
  display.setCursor(115, 20);
  display.println(2-sen1-sen2);
  display.setCursor(0, 40);
  display.println("Slot 1 : ");
  display.setCursor(0, 50);
  display.println("Slot 2 : ");

  display.setCursor(45, 40);
  if (sen1==1){
    display.println("unavaliable"); 
  }
  else{
    display.println("avaliable");
  }

  display.setCursor(45, 50);
  if (sen2==1){
    display.println("unavaliable"); 
  }
  else{
    display.println("avaliable");
  }

  display.display();

  delay(500);

}
 

